# SPDX-FileCopyrightText: {{ year }} Maxwell G <maxwell@gtmx.me>
# SPDX-License-Identifier: MIT
# License text: https://spdx.org/licenses/MIT

Name:           python-{{ config.name }}
Version:        0.0.1
Release:        1%{?dist}
Summary:        {{ config.description }}

License:        MIT
URL:            https://sr.ht/~gotmax23/{{ config.name }}
%global furl    https://git.sr.ht/~gotmax23/{{ config.name }}
Source0:        %{furl}/refs/download/v%{version}/{{ config.name }}-%{version}.tar.gz

BuildArch:      noarch

BuildRequires:  gnupg2
BuildRequires:  python3-devel

%global _description %{expand:
{{ config.description }}.}

%description %_description

%package -n python3-{{ config.name }}
Summary:        %{summary}

%description -n python3-{{ config.name }} %_description


%prep
%autosetup -p1 -n {{ config.name }}-%{version}


%generate_buildrequires
%pyproject_buildrequires -x test


%build
%pyproject_wheel


%install
%pyproject_install
%pyproject_save_files {{ config.name }}


%check
%pytest


%files -n python3-{{ config.name }} -f %{pyproject_files}
%doc README.md
%license LICENSES/*


%changelog

{#
# vim: ft=spec.jinja2:
#}
